; Phone blocker
        LIST P=12C508
        __config _ExtRC_OSC & _WDT_ON & _MCLRE_OFF & _CP_OFF
        Include "p12c508.inc"
; Constant
HandUpTime   EQU    0x4B ; 0,75 second
HandDnTime   EQU    0x32 ; 0,5 second
BlockTimeHi  EQU    0xF8 ; ~20c
MinPulseTime EQU    0x3  ; ~0,03 second
MinPauseTime EQU    0x14 ; ~0,2 second
BlockedNum   EQU    0x8
OrderNumBl   EQU    0x1

;Parols
ParolNumbers EQU    0x3
Parol1       EQU    0x1
Parol2       EQU    0x2
Parol3       EQU    0x3

;Memory Vars
Temp        EQU    0x0F
RingOffCnt  EQU    0x10
HandUpCnt   EQU    0x11
HandDnCnt   EQU    0x12
PulseDetCnt EQU    0x13
ParolOkFlag EQU    0x14
PauseCnt    EQU    0x15
PulseCnt    EQU    0x16
NumberCnt   EQU    0x17
LastLineState EQU  0x18
BtPauseDet  EQU    0x19
ParolNumCnt EQU    0x1A
BlockCntHi  EQU    0x1B
BlockCntLo  EQU    0x1C

;GP0 - Block Line Output / 1-Block Line/ 0-unblock/
;GP1 - Ring Input /1-Ring /0-pause
;GP2 - Hand Up detector /1-Hand down /0-Hand Up
;GP3 - Not Use
;GP4 - When HandUp wating mode - Input /1-My Phone Up / 0-Outdoor Phone Up
;    - When HandDown wating mode - Pulse Input / 1-pause / 0 - pusle
;GP5 - ExtRC

BlockOut      EQU 0
RingIn        EQU 1
LineInH       EQU 2
MyPhoneIn     EQU 4
PulseIn       EQU 4
;---
;        ORG     1FFH
;        movlw   0C0h
        ORG     0
        goto    ReInit       ;
ParolTable:
        andlw   0x03         ;
        addwf   PCL,F        ;
        retlw   Parol3       ;
        retlw   Parol2       ;
        retlw   Parol1       ;
        retlw   0x00         ;
ReInit
        movlw   B'00011110'
        tris    GPIO         ; GP0 - as Out, GP1,2,3,4 - as In
        movlw   B'11001111'  ; No pull-ups, pre to WDT 1:128
        option               ;
        clrf    GPIO         ; GP0 = 0, Line is not block

        clrf    HandDnCnt    ; If !=0 then wait HandDown
        movlw   HandUpTime   ; 
        movwf   HandUpCnt    ; if !=0 then wait HandUp
        clrf    BlockCntHi   ; 
        clrf    BlockCntLo   ; if == 0 then Line is not blocked
        clrf    ParolOkFlag  ; Parol Catcher
        clrf    RingOffCnt   ; RingCnt = 0
        clrf    NumberCnt    ; OrderNumberCnt = 0
        clrf    ParolNumCnt  ;
        clrf    TMR0         ;
MainLoop
        movlw   B'00011110'
        tris    GPIO         ; GP0 - as Out, GP1,2,3,4 - as In
        movlw   0x80         ; 
        andwf   TMR0,W       ; Check TMR0 for greater 80H
        btfss   STATUS,Z     ;
        goto    MainLoop     ; TMR0 is not overflow
;Freq100Hz:
        movlw   0x9D         ; ~100 Hz freq Timer Overflow
        movwf   TMR0         ; Reload TMR0
        clrwdt               ;
        movf    GPIO,W       ; Get currient port state
        movwf   Temp         ; and save to Temp
        
;Check for Blocked line
        movf    BlockCntHi,W ;
        iorwf   BlockCntLo,W ;
        btfss   STATUS,Z     ;
        goto    BlockedLineSrv;        

;Check for "wait HandDown mode"
        movf    HandDnCnt,F  ;
        btfss   STATUS,Z     ; if ==0 then wait HandUpMode
        goto    WaitHandDown ;

;"Wain HandUp mode"
;Check  Ring
        movlw   0xFF         ; 
        btfsc   Temp,RingIn  ; 
        movwf   RingOffCnt   ; If ring in,then RingOffCnt = 255

        movf    RingOffCnt,F ; Test RingOffCnt for 0
        btfsc   STATUS,Z     ; Skip, if != 0
        goto    WaitHandUp   ; 
        decf    RingOffCnt,F ; Dec RingCnt
        movlw   HandUpTime   ; 
        movwf   HandUpCnt    ;
        goto    MainLoop     ; Wait for ring is out 

;No input Ring
;Wait HandUp
WaitHandUp
        movlw   HandUpTime   ; 
        btfsc   Temp,LineInH ; Skip, if LineIn == 0 (HandUp)
        movwf   HandUpCnt    ; If no HandUp,then Set HandUpCnt
        
        decf    HandUpCnt,F  ;
        btfss   STATUS,Z     ; Skip, HandUp == 0, (HandUp Time compile)
        goto    MainLoop     ; if !=0, then time in not complite

;Hand is Up        
;Check for MyPhone
        btfss   Temp,MyPhoneIn; Skip, if MyPhone HandUp
        goto    BlockLine    ;

;Enable "Wait HandDown Mode"
        movlw   HandDnTime   ; 
        movwf   HandDnCnt    ; Wait HandDn
        clrf    PauseCnt     ; Disable Pause Catcher
        clrf    PulseCnt     ; PulseCnt = 0
        clrf    PulseDetCnt  ;
        movlw   0x01         ;
        movwf   LastLineState; Line = 1 (pulse)
        movwf   BtPauseDet   ; 
        incf    BtPauseDet,F ; = 2
        clrf    ParolNumCnt  ;
        goto    MainLoop     ; 

;Pulse nubmer detector
WaitHandDown
;NumberCatcher
        movlw   HandDnTime   ; If no HandDown,then Set HandDnCnt
        btfsc   Temp,PulseIn ; Skip, if PulseIn == 0 (HandDown)
        movwf   HandDnCnt    ; If LineIn = 0 (no HandDown),then Set HandDnCnt
        decfsz  HandDnCnt,F  ; Dec Cnt, 
        goto    NumberAlalyze; 
        goto    ReInit       ; If ==0  (Hand is down)

NumberAlalyze
;Pause Catcher
        movf    PauseCnt,F   ; Check for Enable Pause Catcher
        btfsc   STATUS,Z     ; if !=0 then Catch Pause Enable
        goto    WaitPulse    ;
        movlw   MinPauseTime ; 
        btfss   Temp,PulseIn ; Skip, if PulseIn == 1 (Pause)
        movwf   PauseCnt     ; If LineIn = 1 (Pulse),then Set PauseCnt = Min
        decfsz  PauseCnt,F   ; Dec PauseCnt, 
        goto    WaitPulse    ;
PauseCatched
        movf    ParolNumCnt,F; Check for Parol mode
        btfss   STATUS,Z     ; if == 0 then no Parol mode        
        goto    CheckParol   ; parol mode
CheckNumber
        incf    NumberCnt,F  ; OrderNumberCnt++
        movf    NumberCnt,W  ;
        xorlw   OrderNumBl   ; Order Number for block
        btfss   STATUS,Z     ; Skip, if equ numbers
        goto    NoBlock      ;
;Check for Number        
        movlw   BlockedNum   ;
        xorwf   PulseCnt,W   ;
        btfss   STATUS,Z     ; Skip, if equ numbers
        goto    NoBlock      ;
;ParolLine       
        movlw   ParolNumbers ;
        movwf   ParolNumCnt  ;
        movlw   0x01         ; 
        iorwf   GPIO,F       ; Block Line
        movwf   ParolOkFlag  ; Parol Cacher
NoBlock
        clrf    PulseCnt     ;
        clrf    PauseCnt     ; Disable Pause Catcher
        clrf    PulseDetCnt
        movlw   0x01         ;
        movwf   LastLineState; Line = 1 (pulse)
        movwf   BtPauseDet   ; 
        incf    BtPauseDet,F ; = 2
        goto    MainLoop     ; 

CheckParol
        decf    ParolNumCnt,W;
        call    ParolTable   ; Get Parol for this Number        
;Check for Number        
        xorwf   PulseCnt,W   ;
        btfss   STATUS,Z     ; Skip, if equ numbers
        clrf    ParolOkFlag  ; Bad Parol
;ParolOk        
        decf    ParolNumCnt,F;
        btfss   STATUS,Z     ; Skip, if not all parol enters
        goto    NoBlock      ;
;Unblock Line, If parol Ok
        movf    ParolOkFlag,F ;
        btfsc   STATUS,Z     ; Skip, if parol enter right
        goto    NoBlock      ;
        movlw   0xFE         ; 
        andwf   GPIO,F       ; UnBlock Line
        goto    NoBlock      ;

WaitPulse
        btfss   Temp,PulseIn ; Skip, if PulseIn == 1 (Pause)
        goto    PulseAnalize ;
        movlw   MinPulseTime ; Reload MinPulseTime
        movwf   PulseDetCnt  ;
;Pause between pulses        
        movf   BtPauseDet,F  ; Check for 0
        btfsc  STATUS,Z      ; Skip, if not detect pause between pulses
        goto   BTPauseDetect ;
        decf   BtPauseDet,F  ; Pause between pulses detect Delay 
        goto   MainLoop      ;
BTPauseDetect
        clrf    LastLineState;
        goto    MainLoop     ; 

PulseAnalize
        movf   PulseDetCnt,F ; Check for 0
        btfsc  STATUS,Z      ; Skip, if not detect pulse
        goto   PulseDetect   ; 
        decf   PulseDetCnt,F ; Pulse detect Delay--
        goto   MainLoop      ;
PulseDetect:
        movlw   MinPauseTime ; 
        movwf   PauseCnt     ; Enable Pause Catcher
        movf    LastLineState,F ; Check for Pause in last loop
        btfss   STATUS,Z     ; Skip, if New Pulse detected
        goto    MainLoop     ; 
;NewPulse
        incf    LastLineState,F; LastLineState = 1
        incf    PulseCnt,F   ; PulseCnt++
        incf    BtPauseDet,F ; 
        incf    BtPauseDet,F ; BtPauseDet = 2
        goto    MainLoop     ; 
;****** 

BlockedLineSrv
        incfsz  BlockCntLo,F ; Dec BlockCnt to 0.
        goto    MainLoop     ; 
        incfsz  BlockCntHi,F ;
        goto    MainLoop     ; 
        goto    ReInit       ; 

BlockLine
        movlw   0x01         ; 
        iorwf   GPIO,F       ; Block Line
        clrf    BlockCntLo   ;
        movlw   BlockTimeHi  ; 
        movwf   BlockCntHi   ;
        goto    MainLoop     ; Wait for BlockCnt == 0
	END
